@@ -1,50 +1,15 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2015 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 #import <Foundation/Foundation.h>
 
-// This CPP symbol can be defined to use imports that match up to the framework
-// imports needed when using CocoaPods.
-#if !defined(GPB_USE_PROTOBUF_FRAMEWORK_IMPORTS)
- #define GPB_USE_PROTOBUF_FRAMEWORK_IMPORTS 0
-#endif
-
-#if GPB_USE_PROTOBUF_FRAMEWORK_IMPORTS
- #import <Protobuf/Any.pbobjc.h>
- #import <Protobuf/Duration.pbobjc.h>
- #import <Protobuf/Timestamp.pbobjc.h>
-#else
- #import "google/protobuf/Any.pbobjc.h"
- #import "google/protobuf/Duration.pbobjc.h"
- #import "google/protobuf/Timestamp.pbobjc.h"
-#endif
+#import "GPBAny.pbobjc.h"
+#import "GPBDuration.pbobjc.h"
+#import "GPBTimestamp.pbobjc.h"
 
 NS_ASSUME_NONNULL_BEGIN
 
@@ -153,8 +118,7 @@ typedef NS_ENUM(NSInteger, GPBWellKnownTypesErrorCode) {
  *
  * @return A newly configured GPBAny with the given message, or nil on failure.
  */
-+ (nullable instancetype)anyWithMessage:(nonnull GPBMessage *)message
-                                  error:(NSError **)errorPtr;
++ (nullable instancetype)anyWithMessage:(nonnull GPBMessage *)message error:(NSError **)errorPtr;
 
 /**
  * Convenience method to create a GPBAny containing the serialized message.
@@ -180,8 +144,7 @@ typedef NS_ENUM(NSInteger, GPBWellKnownTypesErrorCode) {
  *
  * @return A newly configured GPBAny with the given message, or nil on failure.
  */
-- (nullable instancetype)initWithMessage:(nonnull GPBMessage *)message
-                                   error:(NSError **)errorPtr;
+- (nullable instancetype)initWithMessage:(nonnull GPBMessage *)message error:(NSError **)errorPtr;
 
 /**
  * Initializes a GPBAny to contain the serialized message.
@@ -207,8 +170,7 @@ typedef NS_ENUM(NSInteger, GPBWellKnownTypesErrorCode) {
  *
  * @return Whether the packing was successful or not.
  */
-- (BOOL)packWithMessage:(nonnull GPBMessage *)message
-                  error:(NSError **)errorPtr;
+- (BOOL)packWithMessage:(nonnull GPBMessage *)message error:(NSError **)errorPtr;
 
 /**
  * Packs the serialized message into this GPBAny.
@@ -237,8 +199,7 @@ typedef NS_ENUM(NSInteger, GPBWellKnownTypesErrorCode) {
  * @return An instance of the given class populated with the contained data, or
  *         nil on failure.
  */
-- (nullable GPBMessage *)unpackMessageClass:(Class)messageClass
-                                      error:(NSError **)errorPtr;
+- (nullable GPBMessage *)unpackMessageClass:(Class)messageClass error:(NSError **)errorPtr;
 
 @end
 
