@@ -1,38 +1,15 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 // Author: jschorr@google.com (Joseph Schorr)
 //  Based on original Protocol Buffers design by
 //  Sanjay Ghemawat, Jeff Dean, and others.
 
-#include <google/protobuf/text_format.h>
+#include "google/protobuf/text_format.h"
 
 #include <math.h>
 #include <stdlib.h>
@@ -40,64 +17,66 @@
 #include <atomic>
 #include <limits>
 #include <memory>
+#include <string>
 
-#include <google/protobuf/stubs/logging.h>
-#include <google/protobuf/stubs/common.h>
-#include <google/protobuf/testing/file.h>
-#include <google/protobuf/testing/file.h>
-#include <google/protobuf/any.pb.h>
-#include <google/protobuf/map_unittest.pb.h>
-#include <google/protobuf/unittest.pb.h>
-#include <google/protobuf/unittest_mset.pb.h>
-#include <google/protobuf/unittest_mset_wire_format.pb.h>
-#include <google/protobuf/unittest_proto3.pb.h>
-#include <google/protobuf/io/tokenizer.h>
-#include <google/protobuf/io/zero_copy_stream_impl.h>
-#include <google/protobuf/stubs/strutil.h>
+#include "google/protobuf/testing/file.h"
+#include "google/protobuf/testing/file.h"
+#include "google/protobuf/any.pb.h"
 #include <gmock/gmock.h>
-#include <google/protobuf/testing/googletest.h>
+#include "google/protobuf/testing/googletest.h"
 #include <gtest/gtest.h>
-#include <google/protobuf/stubs/logging.h>
-#include <google/protobuf/stubs/substitute.h>
-#include <google/protobuf/test_util.h>
-#include <google/protobuf/test_util2.h>
+#include "absl/log/absl_check.h"
+#include "absl/log/die_if_null.h"
+#include "absl/log/scoped_mock_log.h"
+#include "absl/strings/cord.h"
+#include "absl/strings/escaping.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
+#include "absl/strings/str_replace.h"
+#include "absl/strings/substitute.h"
+#include "google/protobuf/io/tokenizer.h"
+#include "google/protobuf/io/zero_copy_stream_impl.h"
+#include "google/protobuf/map_unittest.pb.h"
+#include "google/protobuf/message.h"
+#include "google/protobuf/test_util.h"
+#include "google/protobuf/test_util2.h"
+#include "google/protobuf/unittest.pb.h"
+#include "google/protobuf/unittest_mset.pb.h"
+#include "google/protobuf/unittest_mset_wire_format.pb.h"
+#include "google/protobuf/unittest_proto3.pb.h"
 
 
 // Must be included last.
-#include <google/protobuf/port_def.inc>
+#include "google/protobuf/port_def.inc"
 
 namespace google {
 namespace protobuf {
 
-namespace internal {
-// Controls insertion of DEBUG_STRING_SILENT_MARKER.
-extern PROTOBUF_EXPORT std::atomic<bool> enable_debug_text_format_marker;
-}  // namespace internal
-
 // Can't use an anonymous namespace here due to brokenness of Tru64 compiler.
 namespace text_format_unittest {
 
 using ::google::protobuf::internal::kDebugStringSilentMarker;
+using ::testing::AllOf;
+using ::testing::HasSubstr;
 
 // A basic string with different escapable characters for testing.
-const std::string kEscapeTestString =
+constexpr absl::string_view kEscapeTestString =
     "\"A string with ' characters \n and \r newlines and \t tabs and \001 "
     "slashes \\ and  multiple   spaces";
 
 // A representation of the above string with all the characters escaped.
-const std::string kEscapeTestStringEscaped =
+constexpr absl::string_view kEscapeTestStringEscaped =
     "\"\\\"A string with \\' characters \\n and \\r newlines "
     "and \\t tabs and \\001 slashes \\\\ and  multiple   spaces\"";
 
 class TextFormatTest : public testing::Test {
  public:
   static void SetUpTestSuite() {
-    GOOGLE_CHECK_OK(File::GetContents(
+    ABSL_CHECK_OK(File::GetContents(
         TestUtil::GetTestDataPath(
-            "net/proto2/internal/"
+            "google/protobuf/"
             "testdata/text_format_unittest_data_oneof_implemented.txt"),
         &static_proto_text_format_, true));
-    CleanStringLineEndings(&static_proto_text_format_, false);
   }
 
   TextFormatTest() : proto_text_format_(static_proto_text_format_) {}
@@ -115,11 +94,10 @@ std::string TextFormatTest::static_proto_text_format_;
 class TextFormatExtensionsTest : public testing::Test {
  public:
   static void SetUpTestSuite() {
-    GOOGLE_CHECK_OK(File::GetContents(
-        TestUtil::GetTestDataPath("net/proto2/internal/testdata/"
+    ABSL_CHECK_OK(File::GetContents(
+        TestUtil::GetTestDataPath("google/protobuf/testdata/"
                                   "text_format_unittest_extensions_data.txt"),
         &static_proto_text_format_, true));
-    CleanStringLineEndings(&static_proto_text_format_, false);
   }
 
   TextFormatExtensionsTest() : proto_text_format_(static_proto_text_format_) {}
@@ -155,13 +133,142 @@ TEST_F(TextFormatTest, ShortDebugString) {
   proto_.mutable_optional_foreign_message();
 
   EXPECT_EQ(proto_.ShortDebugString(),
-            StrCat("optional_int32: ", kDebugStringSilentMarker,
+            absl::StrCat("optional_int32: ", kDebugStringSilentMarker,
                          "1 "
                          "optional_string: \"hello\" "
                          "optional_nested_message { bb: 2 } "
                          "optional_foreign_message { }"));
 }
 
+
+TEST_F(TextFormatTest, ShortFormat) {
+  unittest::RedactedFields proto;
+  unittest::TestNestedMessageRedaction redacted_nested_proto;
+  unittest::TestNestedMessageRedaction unredacted_nested_proto;
+  redacted_nested_proto.set_optional_unredacted_nested_string("hello");
+  unredacted_nested_proto.set_optional_unredacted_nested_string("world");
+  proto.set_optional_redacted_string("foo");
+  proto.set_optional_unredacted_string("bar");
+  proto.add_repeated_redacted_string("1");
+  proto.add_repeated_redacted_string("2");
+  proto.add_repeated_unredacted_string("3");
+  proto.add_repeated_unredacted_string("4");
+  *proto.mutable_optional_redacted_message() = redacted_nested_proto;
+  *proto.mutable_optional_unredacted_message() = unredacted_nested_proto;
+  unittest::TestNestedMessageRedaction* redacted_message_1 =
+      proto.add_repeated_redacted_message();
+  unittest::TestNestedMessageRedaction* redacted_message_2 =
+      proto.add_repeated_redacted_message();
+  unittest::TestNestedMessageRedaction* unredacted_message_1 =
+      proto.add_repeated_unredacted_message();
+  unittest::TestNestedMessageRedaction* unredacted_message_2 =
+      proto.add_repeated_unredacted_message();
+  redacted_message_1->set_optional_unredacted_nested_string("5");
+  redacted_message_2->set_optional_unredacted_nested_string("6");
+  unredacted_message_1->set_optional_unredacted_nested_string("7");
+  unredacted_message_2->set_optional_unredacted_nested_string("8");
+  (*proto.mutable_map_redacted_string())["abc"] = "def";
+  (*proto.mutable_map_unredacted_string())["ghi"] = "jkl";
+
+  std::string value_replacement = "\\[REDACTED\\]";
+  EXPECT_THAT(google::protobuf::ShortFormat(proto),
+              testing::MatchesRegex(
+                  "optional_redacted_string: " +
+                  value_replacement +
+                  " "
+                  "optional_unredacted_string: \"bar\" "
+                  "repeated_redacted_string: " +
+                  value_replacement +
+                  " "
+                  "repeated_redacted_string: " +
+                  value_replacement +
+                  " "
+                  "repeated_unredacted_string: \"3\" "
+                  "repeated_unredacted_string: \"4\" "
+                  "optional_redacted_message: " +
+                  value_replacement +
+                  " "
+                  "optional_unredacted_message \\{ "
+                  "optional_unredacted_nested_string: \"world\" \\} "
+                  "repeated_redacted_message: " +
+                  value_replacement +
+                  " "
+                  "repeated_unredacted_message "
+                  "\\{ optional_unredacted_nested_string: \"7\" \\} "
+                  "repeated_unredacted_message "
+                  "\\{ optional_unredacted_nested_string: \"8\" \\} "
+                  "map_redacted_string: " +
+                  value_replacement +
+                  " "
+                  "map_unredacted_string \\{ key: \"ghi\" value: \"jkl\" \\}"));
+}
+
+TEST_F(TextFormatTest, Utf8Format) {
+  unittest::RedactedFields proto;
+  unittest::TestNestedMessageRedaction redacted_nested_proto;
+  unittest::TestNestedMessageRedaction unredacted_nested_proto;
+  redacted_nested_proto.set_optional_unredacted_nested_string(
+      "\350\260\267\346\255\214");
+  unredacted_nested_proto.set_optional_unredacted_nested_string(
+      "\350\260\267\346\255\214");
+  proto.set_optional_redacted_string("foo");
+  proto.set_optional_unredacted_string("bar");
+  proto.add_repeated_redacted_string("1");
+  proto.add_repeated_redacted_string("2");
+  proto.add_repeated_unredacted_string("3");
+  proto.add_repeated_unredacted_string("4");
+  *proto.mutable_optional_redacted_message() = redacted_nested_proto;
+  *proto.mutable_optional_unredacted_message() = unredacted_nested_proto;
+  unittest::TestNestedMessageRedaction* redacted_message_1 =
+      proto.add_repeated_redacted_message();
+  unittest::TestNestedMessageRedaction* redacted_message_2 =
+      proto.add_repeated_redacted_message();
+  unittest::TestNestedMessageRedaction* unredacted_message_1 =
+      proto.add_repeated_unredacted_message();
+  unittest::TestNestedMessageRedaction* unredacted_message_2 =
+      proto.add_repeated_unredacted_message();
+  redacted_message_1->set_optional_unredacted_nested_string("5");
+  redacted_message_2->set_optional_unredacted_nested_string("6");
+  unredacted_message_1->set_optional_unredacted_nested_string("7");
+  unredacted_message_2->set_optional_unredacted_nested_string("8");
+  (*proto.mutable_map_redacted_string())["abc"] = "def";
+  (*proto.mutable_map_unredacted_string())["ghi"] = "jkl";
+
+  std::string value_replacement = "\\[REDACTED\\]";
+  EXPECT_THAT(google::protobuf::Utf8Format(proto),
+              testing::MatchesRegex(
+                  "optional_redacted_string: " +
+                  value_replacement +
+                  "\n"
+                  "optional_unredacted_string: \"bar\"\n"
+                  "repeated_redacted_string: " +
+                  value_replacement +
+                  "\n"
+                  "repeated_redacted_string: " +
+                  value_replacement +
+                  "\n"
+                  "repeated_unredacted_string: \"3\"\n"
+                  "repeated_unredacted_string: \"4\"\n"
+                  "optional_redacted_message: " +
+                  value_replacement +
+                  "\n"
+                  "optional_unredacted_message \\{\n  "
+                  "optional_unredacted_nested_string: "
+                  "\"\xE8\xB0\xB7\xE6\xAD\x8C\"\n\\}\n"
+                  "repeated_redacted_message: " +
+                  value_replacement +
+                  "\n"
+                  "repeated_unredacted_message \\{\n  "
+                  "optional_unredacted_nested_string: \"7\"\n\\}\n"
+                  "repeated_unredacted_message \\{\n  "
+                  "optional_unredacted_nested_string: \"8\"\n\\}\n"
+                  "map_redacted_string: " +
+                  value_replacement +
+                  "\n"
+                  "map_unredacted_string \\{\n  "
+                  "key: \"ghi\"\n  value: \"jkl\"\n\\}\n"));
+}
+
 TEST_F(TextFormatTest, ShortPrimitiveRepeateds) {
   proto_.set_optional_int32(123);
   proto_.add_repeated_int32(456);
@@ -229,7 +336,7 @@ TEST_F(TextFormatTest, StringEscape) {
 
   // Hardcode a correct value to test against.
   std::string correct_string =
-      StrCat("optional_string: ", kDebugStringSilentMarker,
+      absl::StrCat("optional_string: ", kDebugStringSilentMarker,
                    kEscapeTestStringEscaped, "\n");
 
   // Compare.
@@ -238,7 +345,7 @@ TEST_F(TextFormatTest, StringEscape) {
   // the protocol buffer contains no UTF-8 text.
   EXPECT_EQ(correct_string, utf8_debug_string);
 
-  std::string expected_short_debug_string = StrCat(
+  std::string expected_short_debug_string = absl::StrCat(
       "optional_string: ", kDebugStringSilentMarker, kEscapeTestStringEscaped);
   EXPECT_EQ(expected_short_debug_string, proto_.ShortDebugString());
 }
@@ -254,11 +361,11 @@ TEST_F(TextFormatTest, Utf8DebugString) {
 
   // Hardcode a correct value to test against.
   std::string correct_utf8_string =
-      StrCat("optional_string: ", kDebugStringSilentMarker,
+      absl::StrCat("optional_string: ", kDebugStringSilentMarker,
                    "\"\350\260\267\346\255\214\"\n"
                    "optional_bytes: \"\\350\\260\\267\\346\\255\\214\"\n");
   std::string correct_string =
-      StrCat("optional_string: ", kDebugStringSilentMarker,
+      absl::StrCat("optional_string: ", kDebugStringSilentMarker,
                    "\"\\350\\260\\267\\346\\255\\214\"\n"
                    "optional_bytes: \"\\350\\260\\267\\346\\255\\214\"\n");
 
@@ -283,7 +390,7 @@ TEST_F(TextFormatTest, PrintUnknownFields) {
   unknown_fields->AddVarint(8, 2);
   unknown_fields->AddVarint(8, 3);
 
-  EXPECT_EQ(StrCat("5: ", kDebugStringSilentMarker,
+  EXPECT_EQ(absl::StrCat("5: ", kDebugStringSilentMarker,
                          "1\n"
                          "5: 0x00000002\n"
                          "5: 0x0000000000000003\n"
@@ -297,6 +404,22 @@ TEST_F(TextFormatTest, PrintUnknownFields) {
             message.DebugString());
 }
 
+TEST_F(TextFormatTest, PrintUnknownFieldsDeepestStackWorks) {
+  // Test printing of unknown fields in a message.
+
+  unittest::TestEmptyMessage message;
+  UnknownFieldSet* unknown_fields = message.mutable_unknown_fields();
+
+  for (int i = 0; i < 200; ++i) {
+    unknown_fields = unknown_fields->AddGroup(1);
+  }
+
+  unknown_fields->AddVarint(2, 100);
+
+  std::string str;
+  EXPECT_TRUE(TextFormat::PrintToString(message, &str));
+}
+
 TEST_F(TextFormatTest, PrintUnknownFieldsHidden) {
   // Test printing of unknown fields in a message when suppressed.
 
@@ -459,7 +582,7 @@ TEST_F(TextFormatTest, PrintBufferTooSmall) {
 class CustomUInt32FieldValuePrinter : public TextFormat::FieldValuePrinter {
  public:
   std::string PrintUInt32(uint32_t val) const override {
-    return StrCat(FieldValuePrinter::PrintUInt32(val), "u");
+    return absl::StrCat(FieldValuePrinter::PrintUInt32(val), "u");
   }
 };
 
@@ -483,7 +606,7 @@ TEST_F(TextFormatTest, DefaultCustomFieldPrinter) {
 class CustomInt32FieldValuePrinter : public TextFormat::FieldValuePrinter {
  public:
   std::string PrintInt32(int32_t val) const override {
-    return StrCat("value-is(", FieldValuePrinter::PrintInt32(val), ")");
+    return absl::StrCat("value-is(", FieldValuePrinter::PrintInt32(val), ")");
   }
 };
 
@@ -534,8 +657,8 @@ TEST_F(TextFormatTest, ErrorCasesRegisteringFieldValuePrinterShouldFail) {
 class CustomMessageFieldValuePrinter : public TextFormat::FieldValuePrinter {
  public:
   std::string PrintInt32(int32_t v) const override {
-    return StrCat(FieldValuePrinter::PrintInt32(v), "  # x",
-                        strings::Hex(v));
+    return absl::StrCat(FieldValuePrinter::PrintInt32(v), "  # x",
+                        absl::Hex(v));
   }
 
   std::string PrintMessageStart(const Message& message, int field_index,
@@ -544,7 +667,7 @@ class CustomMessageFieldValuePrinter : public TextFormat::FieldValuePrinter {
     if (single_line_mode) {
       return " { ";
     }
-    return StrCat(" {  # ", message.GetDescriptor()->name(), ": ",
+    return absl::StrCat(" {  # ", message.GetDescriptor()->name(), ": ",
                         field_index, "\n");
   }
 };
@@ -590,7 +713,7 @@ class CustomMessageContentFieldValuePrinter
       TextFormat::BaseTextGenerator* generator) const override {
     if (message.ByteSizeLong() > 0) {
       generator->PrintString(
-          strings::Substitute("# REDACTED, $0 bytes\n", message.ByteSizeLong()));
+          absl::Substitute("# REDACTED, $0 bytes\n", message.ByteSizeLong()));
     }
     return true;
   }
@@ -634,7 +757,7 @@ class CustomMultilineCommentPrinter : public TextFormat::FieldValuePrinter {
   std::string PrintMessageStart(const Message& message, int field_index,
                                 int field_count,
                                 bool single_line_comment) const override {
-    return StrCat(" {  # 1\n", "  # 2\n");
+    return absl::StrCat(" {  # 1\n", "  # 2\n");
   }
 };
 
@@ -780,15 +903,24 @@ class CustomNestedMessagePrinter : public TextFormat::MessagePrinter {
   ~CustomNestedMessagePrinter() override {}
   void Print(const Message& message, bool single_line_mode,
              TextFormat::BaseTextGenerator* generator) const override {
-    generator->PrintLiteral("custom");
+    generator->PrintLiteral("// custom\n");
+    if (printer_ != nullptr) {
+      printer_->PrintMessage(message, generator);
+    }
   }
+
+  void SetPrinter(TextFormat::Printer* printer) { printer_ = printer; }
+
+ private:
+  TextFormat::Printer* printer_ = nullptr;
 };
 
 TEST_F(TextFormatTest, CustomMessagePrinter) {
   TextFormat::Printer printer;
+  auto* custom_printer = new CustomNestedMessagePrinter;
   printer.RegisterMessagePrinter(
       unittest::TestAllTypes::NestedMessage::default_instance().descriptor(),
-      new CustomNestedMessagePrinter);
+      custom_printer);
 
   unittest::TestAllTypes message;
   std::string text;
@@ -797,7 +929,11 @@ TEST_F(TextFormatTest, CustomMessagePrinter) {
 
   message.mutable_optional_nested_message()->set_bb(1);
   EXPECT_TRUE(printer.PrintToString(message, &text));
-  EXPECT_EQ("optional_nested_message {\n  custom}\n", text);
+  EXPECT_EQ("optional_nested_message {\n  // custom\n}\n", text);
+
+  custom_printer->SetPrinter(&printer);
+  EXPECT_TRUE(printer.PrintToString(message, &text));
+  EXPECT_EQ("optional_nested_message {\n  // custom\n  bb: 1\n}\n", text);
 }
 
 TEST_F(TextFormatTest, ParseBasic) {
@@ -807,6 +943,12 @@ TEST_F(TextFormatTest, ParseBasic) {
   TestUtil::ExpectAllFieldsSet(proto_);
 }
 
+TEST_F(TextFormatTest, ParseCordBasic) {
+  absl::Cord cord(proto_text_format_);
+  TextFormat::ParseFromCord(cord, &proto_);
+  TestUtil::ExpectAllFieldsSet(proto_);
+}
+
 TEST_F(TextFormatExtensionsTest, ParseExtensions) {
   io::ArrayInputStream input_stream(proto_text_format_.data(),
                                     proto_text_format_.size());
@@ -817,7 +959,7 @@ TEST_F(TextFormatExtensionsTest, ParseExtensions) {
 TEST_F(TextFormatTest, ParseEnumFieldFromNumber) {
   // Create a parse string with a numerical value for an enum field.
   std::string parse_string =
-      strings::Substitute("optional_nested_enum: $0", unittest::TestAllTypes::BAZ);
+      absl::Substitute("optional_nested_enum: $0", unittest::TestAllTypes::BAZ);
   EXPECT_TRUE(TextFormat::ParseFromString(parse_string, &proto_));
   EXPECT_TRUE(proto_.has_optional_nested_enum());
   EXPECT_EQ(unittest::TestAllTypes::BAZ, proto_.optional_nested_enum());
@@ -826,7 +968,7 @@ TEST_F(TextFormatTest, ParseEnumFieldFromNumber) {
 TEST_F(TextFormatTest, ParseEnumFieldFromNegativeNumber) {
   ASSERT_LT(unittest::SPARSE_E, 0);
   std::string parse_string =
-      strings::Substitute("sparse_enum: $0", unittest::SPARSE_E);
+      absl::Substitute("sparse_enum: $0", unittest::SPARSE_E);
   unittest::SparseEnumMessage proto;
   EXPECT_TRUE(TextFormat::ParseFromString(parse_string, &proto));
   EXPECT_TRUE(proto.has_sparse_enum());
@@ -845,7 +987,7 @@ TEST_F(TextFormatTest, PrintUnknownEnumFieldProto3) {
   proto.add_repeated_nested_enum(
       static_cast<proto3_unittest::TestAllTypes::NestedEnum>(-2147483648));
 
-  EXPECT_EQ(StrCat("repeated_nested_enum: ", kDebugStringSilentMarker,
+  EXPECT_EQ(absl::StrCat("repeated_nested_enum: ", kDebugStringSilentMarker,
                          "10\n"
                          "repeated_nested_enum: -10\n"
                          "repeated_nested_enum: 2147483647\n"
@@ -865,10 +1007,93 @@ TEST_F(TextFormatTest, ParseUnknownEnumFieldProto3) {
   EXPECT_EQ(-2147483648, proto.repeated_nested_enum(3));
 }
 
+TEST_F(TextFormatTest, ErrorOnNoOpFieldsProto3) {
+  proto3_unittest::TestAllTypes proto;
+  TextFormat::Parser parser;
+  parser.ErrorOnNoOpFields(true);
+
+  {
+    const absl::string_view singular_int_parse_string = "optional_int32: 0";
+    EXPECT_TRUE(TextFormat::ParseFromString(singular_int_parse_string, &proto));
+    EXPECT_FALSE(parser.ParseFromString(singular_int_parse_string, &proto));
+  }
+  {
+    const absl::string_view singular_bool_parse_string = "optional_bool: false";
+    EXPECT_TRUE(
+        TextFormat::ParseFromString(singular_bool_parse_string, &proto));
+    EXPECT_FALSE(parser.ParseFromString(singular_bool_parse_string, &proto));
+  }
+  {
+    const absl::string_view singular_string_parse_string =
+        "optional_string: ''";
+    EXPECT_TRUE(
+        TextFormat::ParseFromString(singular_string_parse_string, &proto));
+    EXPECT_FALSE(parser.ParseFromString(singular_string_parse_string, &proto));
+  }
+  {
+    const absl::string_view nested_message_parse_string =
+        "optional_nested_message { bb: 0 } ";
+    EXPECT_TRUE(
+        TextFormat::ParseFromString(nested_message_parse_string, &proto));
+    EXPECT_FALSE(parser.ParseFromString(nested_message_parse_string, &proto));
+  }
+  {
+    const absl::string_view foreign_message_parse_string =
+        "optional_foreign_message { c: 0 } ";
+    EXPECT_TRUE(
+        TextFormat::ParseFromString(foreign_message_parse_string, &proto));
+    EXPECT_FALSE(parser.ParseFromString(foreign_message_parse_string, &proto));
+  }
+  {
+    const absl::string_view nested_enum_parse_string =
+        "optional_nested_enum: ZERO ";
+    EXPECT_TRUE(TextFormat::ParseFromString(nested_enum_parse_string, &proto));
+    EXPECT_FALSE(parser.ParseFromString(nested_enum_parse_string, &proto));
+  }
+  {
+    const absl::string_view foreign_enum_parse_string =
+        "optional_foreign_enum: FOREIGN_ZERO ";
+    EXPECT_TRUE(TextFormat::ParseFromString(foreign_enum_parse_string, &proto));
+    EXPECT_FALSE(parser.ParseFromString(foreign_enum_parse_string, &proto));
+  }
+  {
+    const absl::string_view string_piece_parse_string =
+        "optional_string_piece: '' ";
+    EXPECT_TRUE(TextFormat::ParseFromString(string_piece_parse_string, &proto));
+    EXPECT_FALSE(parser.ParseFromString(string_piece_parse_string, &proto));
+  }
+  {
+    const absl::string_view cord_parse_string = "optional_cord: '' ";
+    EXPECT_TRUE(TextFormat::ParseFromString(cord_parse_string, &proto));
+    EXPECT_FALSE(parser.ParseFromString(cord_parse_string, &proto));
+  }
+  {
+    // Sanity check that repeated fields work the same.
+    const absl::string_view repeated_int32_parse_string = "repeated_int32: 0 ";
+    EXPECT_TRUE(
+        TextFormat::ParseFromString(repeated_int32_parse_string, &proto));
+    EXPECT_TRUE(parser.ParseFromString(repeated_int32_parse_string, &proto));
+  }
+  {
+    const absl::string_view repeated_bool_parse_string =
+        "repeated_bool: false  ";
+    EXPECT_TRUE(
+        TextFormat::ParseFromString(repeated_bool_parse_string, &proto));
+    EXPECT_TRUE(parser.ParseFromString(repeated_bool_parse_string, &proto));
+  }
+  {
+    const absl::string_view repeated_string_parse_string =
+        "repeated_string: '' ";
+    EXPECT_TRUE(
+        TextFormat::ParseFromString(repeated_string_parse_string, &proto));
+    EXPECT_TRUE(parser.ParseFromString(repeated_string_parse_string, &proto));
+  }
+}
+
 TEST_F(TextFormatTest, ParseStringEscape) {
   // Create a parse string with escaped characters in it.
   std::string parse_string =
-      "optional_string: " + kEscapeTestStringEscaped + "\n";
+      absl::StrCat("optional_string: ", kEscapeTestStringEscaped, "\n");
 
   io::ArrayInputStream input_stream(parse_string.data(), parse_string.size());
   TextFormat::Parse(&input_stream, &proto_);
@@ -964,6 +1189,7 @@ TEST_F(TextFormatTest, ParseShortRepeatedWithTrailingComma) {
   parse_string = "repeated_nested_message: [ { bb: 1 }, ]";
   ASSERT_FALSE(TextFormat::ParseFromString(parse_string, &proto_));
   parse_string = "RepeatedGroup [{ a: 3 },]\n";
+  ASSERT_FALSE(TextFormat::ParseFromString(parse_string, &proto_));
 }
 
 TEST_F(TextFormatTest, ParseShortRepeatedEmpty) {
@@ -1035,6 +1261,19 @@ TEST_F(TextFormatTest, ParseShortRepeatedConcatenatedWithEmpty) {
   EXPECT_EQ(4, proto_.repeatedgroup(1).a());
 }
 
+TEST_F(TextFormatTest, ParseShortRepeatedUnknownEmpty) {
+  std::string parse_string =
+      "repeated_string: \"before\"\n"
+      "unknown_field: []\n"
+      "repeated_string: \"after\"\n";
+  TextFormat::Parser parser;
+  parser.AllowUnknownField(true);
+
+  ASSERT_TRUE(parser.ParseFromString(parse_string, &proto_));
+
+  EXPECT_EQ(2, proto_.repeated_string_size());
+}
+
 
 TEST_F(TextFormatTest, Comments) {
   // Test that comments are ignored.
@@ -1069,10 +1308,8 @@ TEST_F(TextFormatTest, OptionalColon) {
 
 // Some platforms (e.g. Windows) insist on padding the exponent to three
 // digits when one or two would be just fine.
-static std::string RemoveRedundantZeros(std::string text) {
-  text = StringReplace(text, "e+0", "e+", true);
-  text = StringReplace(text, "e-0", "e-", true);
-  return text;
+static std::string RemoveRedundantZeros(absl::string_view text) {
+  return absl::StrReplaceAll(text, {{"e+0", "e+"}, {"e-0", "e-"}});
 }
 
 TEST_F(TextFormatTest, PrintExotic) {
@@ -1103,7 +1340,7 @@ TEST_F(TextFormatTest, PrintExotic) {
   //   have this problem, so we switched to that instead.
 
   EXPECT_EQ(
-      StrCat("repeated_int64: ", kDebugStringSilentMarker,
+      absl::StrCat("repeated_int64: ", kDebugStringSilentMarker,
                    "-9223372036854775808\n"
                    "repeated_uint64: 18446744073709551615\n"
                    "repeated_double: 123.456\n"
@@ -1163,7 +1400,7 @@ TEST_F(TextFormatTest, PrintFloatPrecision) {
   message.add_repeated_double(1.2345678987654e100);
   message.add_repeated_double(1.23456789876543e100);
 
-  EXPECT_EQ(StrCat("repeated_float: ", kDebugStringSilentMarker,
+  EXPECT_EQ(absl::StrCat("repeated_float: ", kDebugStringSilentMarker,
                          "1\n"
                          "repeated_float: 1.2\n"
                          "repeated_float: 1.23\n"
@@ -1401,7 +1638,7 @@ class TextFormatParserTest : public testing::Test {
     parser_.RecordErrorsTo(&error_collector);
     EXPECT_EQ(expected_result, parser_.ParseFromString(input, proto))
         << input << " -> " << proto->DebugString();
-    EXPECT_EQ(StrCat(line, ":", col, ": ", message, "\n"),
+    EXPECT_EQ(absl::StrCat(line, ":", col, ": ", message, "\n"),
               error_collector.text_);
     parser_.RecordErrorsTo(nullptr);
   }
@@ -1441,13 +1678,14 @@ class TextFormatParserTest : public testing::Test {
     std::string text_;
 
     // implements ErrorCollector -------------------------------------
-    void AddError(int line, int column, const std::string& message) override {
-      strings::SubstituteAndAppend(&text_, "$0:$1: $2\n", line + 1, column + 1,
+    void RecordError(int line, int column, absl::string_view message) override {
+      absl::SubstituteAndAppend(&text_, "$0:$1: $2\n", line + 1, column + 1,
                                 message);
     }
 
-    void AddWarning(int line, int column, const std::string& message) override {
-      AddError(line, column, "WARNING:" + message);
+    void RecordWarning(int line, int column,
+                       absl::string_view message) override {
+      RecordError(line, column, absl::StrCat("WARNING:", message));
     }
   };
 
@@ -1636,7 +1874,7 @@ TEST_F(TextFormatParserTest, ParseFieldValueFromString) {
   // enum
   EXPECT_FIELD(nested_enum, unittest::TestAllTypes::BAR, "BAR");
   EXPECT_FIELD(nested_enum, unittest::TestAllTypes::BAZ,
-               StrCat(unittest::TestAllTypes::BAZ));
+               absl::StrCat(unittest::TestAllTypes::BAZ));
   EXPECT_INVALID(nested_enum, "FOOBAR");
 
   // message
@@ -1859,38 +2097,33 @@ TEST_F(TextFormatParserTest, ExplicitDelimiters) {
 }
 
 TEST_F(TextFormatParserTest, PrintErrorsToStderr) {
-  std::vector<std::string> errors;
-
   {
-    ScopedMemoryLog log;
+    absl::ScopedMockLog log(absl::MockLogDefault::kDisallowUnexpected);
+    EXPECT_CALL(
+        log,
+        Log(absl::LogSeverity::kError, testing::_,
+            "Error parsing text-format protobuf_unittest.TestAllTypes: "
+            "1:14: Message type \"protobuf_unittest.TestAllTypes\" has no field "
+            "named \"no_such_field\"."))
+        .Times(1);
+    log.StartCapturingLogs();
     unittest::TestAllTypes proto;
     EXPECT_FALSE(TextFormat::ParseFromString("no_such_field: 1", &proto));
-    errors = log.GetMessages(ERROR);
   }
-
-  ASSERT_EQ(1, errors.size());
-  EXPECT_EQ(
-      "Error parsing text-format protobuf_unittest.TestAllTypes: "
-      "1:14: Message type \"protobuf_unittest.TestAllTypes\" has no field "
-      "named \"no_such_field\".",
-      errors[0]);
 }
 
 TEST_F(TextFormatParserTest, FailsOnTokenizationError) {
-  std::vector<std::string> errors;
-
   {
-    ScopedMemoryLog log;
+    absl::ScopedMockLog log(absl::MockLogDefault::kDisallowUnexpected);
+    EXPECT_CALL(log,
+                Log(absl::LogSeverity::kError, testing::_,
+                    "Error parsing text-format protobuf_unittest.TestAllTypes: "
+                    "1:1: Invalid control characters encountered in text."))
+        .Times(1);
+    log.StartCapturingLogs();
     unittest::TestAllTypes proto;
     EXPECT_FALSE(TextFormat::ParseFromString("\020", &proto));
-    errors = log.GetMessages(ERROR);
   }
-
-  ASSERT_EQ(1, errors.size());
-  EXPECT_EQ(
-      "Error parsing text-format protobuf_unittest.TestAllTypes: "
-      "1:1: Invalid control characters encountered in text.",
-      errors[0]);
 }
 
 TEST_F(TextFormatParserTest, ParseDeprecatedField) {
@@ -1898,18 +2131,22 @@ TEST_F(TextFormatParserTest, ParseDeprecatedField) {
   ExpectMessage("deprecated_int32: 42",
                 "WARNING:text format contains deprecated field "
                 "\"deprecated_int32\"",
-                1, 21, &message, true);
+                1, 17, &message, true);
+  ExpectMessage("deprecated_message {\n#blah\n#blah\n#blah\n}\n",
+                "WARNING:text format contains deprecated field "
+                "\"deprecated_message\"",
+                1, 20, &message, true);
 }
 
 TEST_F(TextFormatParserTest, SetRecursionLimit) {
   const char* format = "child: { $0 }";
   std::string input;
-  for (int i = 0; i < 100; ++i) input = strings::Substitute(format, input);
+  for (int i = 0; i < 100; ++i) input = absl::Substitute(format, input);
 
   unittest::NestedTestAllTypes message;
   ExpectSuccessAndTree(input, &message, nullptr);
 
-  input = strings::Substitute(format, input);
+  input = absl::Substitute(format, input);
   parser_.SetRecursionLimit(100);
   ExpectMessage(input,
                 "Message is too deep, the parser exceeded the configured "
@@ -1923,8 +2160,8 @@ TEST_F(TextFormatParserTest, SetRecursionLimit) {
 TEST_F(TextFormatParserTest, SetRecursionLimitUnknownFieldValue) {
   const char* format = "[$0]";
   std::string input = "\"test_value\"";
-  for (int i = 0; i < 99; ++i) input = strings::Substitute(format, input);
-  std::string not_deep_input = StrCat("unknown_nested_array: ", input);
+  for (int i = 0; i < 99; ++i) input = absl::Substitute(format, input);
+  std::string not_deep_input = absl::StrCat("unknown_nested_array: ", input);
 
   parser_.AllowUnknownField(true);
   parser_.SetRecursionLimit(100);
@@ -1932,8 +2169,8 @@ TEST_F(TextFormatParserTest, SetRecursionLimitUnknownFieldValue) {
   unittest::NestedTestAllTypes message;
   ExpectSuccessAndTree(not_deep_input, &message, nullptr);
 
-  input = strings::Substitute(format, input);
-  std::string deep_input = StrCat("unknown_nested_array: ", input);
+  input = absl::Substitute(format, input);
+  std::string deep_input = absl::StrCat("unknown_nested_array: ", input);
   ExpectMessage(
       deep_input,
       "WARNING:Message type \"protobuf_unittest.NestedTestAllTypes\" has no "
@@ -1948,7 +2185,7 @@ TEST_F(TextFormatParserTest, SetRecursionLimitUnknownFieldValue) {
 TEST_F(TextFormatParserTest, SetRecursionLimitUnknownFieldMessage) {
   const char* format = "unknown_child: { $0 }";
   std::string input;
-  for (int i = 0; i < 100; ++i) input = strings::Substitute(format, input);
+  for (int i = 0; i < 100; ++i) input = absl::Substitute(format, input);
 
   parser_.AllowUnknownField(true);
   parser_.SetRecursionLimit(100);
@@ -1956,7 +2193,7 @@ TEST_F(TextFormatParserTest, SetRecursionLimitUnknownFieldMessage) {
   unittest::NestedTestAllTypes message;
   ExpectSuccessAndTree(input, &message, nullptr);
 
-  input = strings::Substitute(format, input);
+  input = absl::Substitute(format, input);
   ExpectMessage(
       input,
       "WARNING:Message type \"protobuf_unittest.NestedTestAllTypes\" has no "
@@ -2255,6 +2492,7 @@ TEST(TextFormatUnknownFieldTest, TestUnknownExtension) {
   EXPECT_FALSE(parser.ParseFromString("unknown_field: 1", &proto));
 }
 
+
 TEST(TextFormatFloatingPointTest, PreservesNegative0) {
   proto3_unittest::TestAllTypes in_message;
   in_message.set_optional_float(-0.0f);
@@ -2273,8 +2511,9 @@ TEST(TextFormatFloatingPointTest, PreservesNegative0) {
             std::signbit(out_message.optional_double()));
 }
 
+
 }  // namespace text_format_unittest
 }  // namespace protobuf
 }  // namespace google
 
-#include <google/protobuf/port_undef.inc>
+#include "google/protobuf/port_undef.inc"
